"""
.. module: bozor.exceptions
    :platform: Unix
    :copyright: (c) 2015 by Netflix Inc., see AUTHORS for more
    :license: Apache, see LICENSE for more details.
.. moduleauthor:: Patrick Kelley <patrick@netflix.com>
"""


class BozorException(Exception):
    pass



__all__ = [
    '__title__',
    '__summary__',
    '__uri__',
    '__version__',
    '__author__',
    '__email__',
    '__license__',
    '__copyright__'
]

__title__ = 'bozor'
__summary__ = 'A higher level package to retrieve details on AWS items.'
__uri__ = 'https://github.com/monkeysecurity/bozor'

__version__ = '0.0.1dev0'

__author__ = 'Patrick Kelley'
__email__ = 'patrick@netflix.com'

__license__ = 'Apache License, Version 2.0'
__copyright__ = 'Copyright 2016 %s' % __author__



"""
.. module: bozor.aws.arn
    :platform: Unix
.. version:: $$VERSION$$
.. moduleauthor:: Patrick Kelley <pkelley@netflix.com>
"""

import re


class ARN(object):
    tech = None
    region = None
    account_number = None
    name = None
    parsed_name = None
    partition = None
    error = False
    root = False
    service = False

    def __init__(self, input):
        arn_match = re.search('^arn:([^:]*):([^:]*):([^:]*):(|\*|[\d]{12}):(.+)$', input)
        if arn_match:
            if arn_match.group(2) == "iam" and arn_match.group(5) == "root":
                self.root = True

            return self._from_arn(arn_match, input)

        acct_number_match = re.search('^(\d{12})+$', input)
        if acct_number_match:
            return self._from_account_number(input)

        aws_service_match = re.search('^([^.]*)\.amazonaws\.com$', input)
        if aws_service_match:
            return self._from_aws_service(input, aws_service_match.group(1))

        self.error = True

    def _from_arn(self, arn_match, input):
        self.partition = arn_match.group(1)
        self.tech = arn_match.group(2)
        self.region = arn_match.group(3)
        self.account_number = arn_match.group(4)
        self.name = arn_match.group(5)
        self.parsed_name = self.name.split('/')[-1]

    def _from_account_number(self, input):
        self.account_number = input

    def _from_aws_service(self, input, service):
        self.tech = service
        self.service = True

    @staticmethod
    def extract_arns_from_statement_condition(condition):
        condition_subsection \
            = condition.get('ArnEquals', {}) or \
              condition.get('ForAllValues:ArnEquals', {}) or \
              condition.get('ForAnyValue:ArnEquals', {}) or \
              condition.get('ArnLike', {}) or \
              condition.get('ForAllValues:ArnLike', {}) or \
              condition.get('ForAnyValue:ArnLike', {}) or \
              condition.get('StringLike', {}) or \
              condition.get('ForAllValues:StringLike', {}) or \
              condition.get('ForAnyValue:StringLike', {}) or \
              condition.get('StringEquals', {}) or \
              condition.get('ForAllValues:StringEquals', {}) or \
              condition.get('ForAnyValue:StringEquals', {})

        # aws:sourcearn can be found with in lowercase or camelcase or other cases...
        condition_arns = []
        for key, value in condition_subsection.iteritems():
            if key.lower() == 'aws:sourcearn' or key.lower() == 'aws:sourceowner':
                if isinstance(value, list):
                    condition_arns.extend(value)
                else:
                    condition_arns.append(value)

        if not isinstance(condition_arns, list):
            return [condition_arns]
        return condition_arns




"""
Bozor
=====

A higher level package to retrieve details on AWS items.

:copyright: (c) 2016 by Netflix, see AUTHORS for more
:license: Apache, see LICENSE for more details.
"""
import sys
import os.path

from setuptools import setup, find_packages

ROOT = os.path.realpath(os.path.join(os.path.dirname(__file__)))

# When executing the setup.py, we need to be able to import ourselves.  This
# means that we need to add the src/ directory to the sys.path

sys.path.insert(0, ROOT)

about = {}
with open(os.path.join(ROOT, "bozor", "__about__.py")) as f:
    exec(f.read(), about)

install_requires = [
    'botor>=0.0.1dev7',
    'inflection'
]

tests_require = []

docs_require = []

dev_require = []

setup(
    name=about["__title__"],
    version=about["__version__"],
    author=about["__author__"],
    author_email=about["__email__"],
    url=about["__uri__"],
    description=about["__summary__"],
    long_description=open(os.path.join(ROOT, 'README.md')).read(),
    packages=find_packages(),
    include_package_data=True,
    zip_safe=False,
    install_requires=install_requires,
    extras_require={
        'tests': tests_require,
        'docs': docs_require,
        'dev': dev_require
    }
)



from bozor.exceptions import BozorException
from bozor.aws.arn import ARN

from botor.aws.iam import get_role_inline_policies
from botor.aws.iam import get_role_instance_profiles
from botor.aws.iam import get_role_managed_policies
from botor import Botor

from inflection import camelize, underscore


def _conn_from_arn(arn):
    """
    Extracts the account number from an ARN.
    :param arn: Amazon ARN containing account number.
    :return: dictionary with a single account_number key that can be merged with an existing
    connection dictionary containing fields such as assume_role, session_name, region.
    """
    role_arn = ARN(arn)
    if role_arn.error:
        raise BozorException('Bad ARN: {arn}'.format(arn=arn))
    return dict(
        account_number=role_arn.account_number,
    )


def _get_role_name(role):
    """
    Given a possibly sparsely populated role dictionary, try to retrieve the role name.
    First try the role_name field.  If that doesn't exist, try to parse the role name from
    the ARN.
    :param role: dict containing (at the very least) role_name and/or arn
    :return: role name
    """
    if role.get('RoleName'):
        return role.get('RoleName')

    if role.get('Arn'):
        arn = role.get('Arn')
        role_arn = ARN(arn)
        if role_arn.error:
            raise BozorException('Bad ARN: {arn}'.format(arn=arn))
        return role_arn.parsed_name

    raise BozorException('Cannot extract role name from input: {input}.'.format(input=role))


def _get_base(role, **conn):
    """
    Determine whether the boto get_role call needs to be made or if we already have all that data
    in the role object.
    :param role: dict containing (at the very least) role_name and/or arn.
    :param conn: dict containing enough information to make a connection to the desired account.
    :return: Camelized dict describing role containing all all base_fields.
    """
    base_fields = frozenset(['Arn', 'AssumeRolePolicyDocument', 'Path', 'RoleId', 'RoleName', 'CreateDate'])
    needs_base = False

    for field in base_fields:
        if field not in role:
            needs_base = True
            break

    if needs_base:
        role_name = _get_role_name(role)
        role = Botor.go('iam.client.get_role', RoleName=role_name, **conn)
        role = role['Role']

        # cast CreateDate from a datetime to something JSON serializable.
        role.update(dict(CreateDate=str(role['CreateDate'])))

    return role


def _modify(role, func):
    """
    Modifies each role.keys() string based on the func passed in.
    Often used with inflection's camelize or underscore methods.

    :param role: dictionary representing role to be modified
    :param func: function to run on each key string
    :return: dictionary where each key has been modified by func.
    """
    for key in role:
        new_key = func(key)
        if key != new_key:
            role[new_key] = role[key]
            del role[key]
    return role


def modify(role, format='camelized'):
    """
    Calls _modify and either passes the inflection.camelize method or the inflection.underscore method.

    :param role: dictionary representing role to be modified
    :param format: string 'camelized' or 'underscored'
    :return:
    """
    if format == 'camelized':
        return _modify(role, camelize)
    elif format == 'underscored':
        return _modify(role, underscore)


def get_role(role, output='camelized', **conn):
    """
    Orchestrates all the calls required to fully build out an IAM Role in the following format:

    {
        "Arn": ...,
        "AssumeRolePolicyDocument": ...,
        "CreateDate": ...,  # str
        "InlinePolicies": ...,
        "InstanceProfiles": ...,
        "ManagedPolicies": ...,
        "Path": ...,
        "RoleId": ...,
        "RoleName": ...,
    }

    :param role: dict containing (at the very least) role_name and/or arn.
    :param output: Determines whether keys should be returned camelized or underscored.
    :param conn: dict containing enough information to make a connection to the desired account.
    Must at least have 'assume_role' key.
    :return: dict containing a fully built out role.
    """
    role = modify(role, 'camelized')
    if role.get('Arn'):  # otherwise, account_number can be included in **conn
        conn.update(_conn_from_arn(role.get('Arn')))
    role = _get_base(role, **conn)
    role.update(
        {
            'managed_policies': get_role_managed_policies(role, **conn),
            'inline_policies': get_role_inline_policies(role, **conn),
            'instance_profiles': get_role_instance_profiles(role, **conn)
        }
    )
    return modify(role, format=output)
